DOMAIN_LABELS = [
    "messaging",
    "calling",
    "event",
    "timer",
    "music",
    "weather",
    "alarm",
    "people",
    "reminder",
    "recipes",
    "news",
]

INTENT_LABELS = [
    "SEND_MESSAGE",
    "GET_MESSAGE",
    "GET_MESSAGE_CONTACT",
    "CREATE_CALL",
    "GET_CONTACT",
    "GET_AVAILABILITY",
    "SET_RSVP_INTERESTED",
    "SET_RSVP_YES",
    "SET_DEFAULT_PROVIDER_CALLING",
    "PAUSE_TIMER",
    "GET_SUNSET",
    "GET_TIMER",
    "SUBTRACT_TIME_TIMER",
    "RESUME_TIMER",
    "CREATE_ALARM",
    "CREATE_TIMER",
    "PLAY_MUSIC",
    "SET_DEFAULT_PROVIDER_MUSIC",
    "GET_STORIES_NEWS",
    "UPDATE_TIMER",
    "SHARE_EVENT",
    "PLAY_MEDIA",
    "UPDATE_CALL",
    "STOP_SHUFFLE_MUSIC",
    "GET_ALARM",
    "GET_WEATHER",
    "PREFER",
    "GET_LOCATION",
    "GET_DATE_TIME_EVENT",
    "GET_EVENT",
    "GET_CATEGORY_EVENT",
    "UPDATE_METHOD_CALL",
    "END_CALL",
    "SET_UNAVAILABLE",
    "GET_CALL_TIME",
    "GET_CALL",
    "SNOOZE_ALARM",
    "SILENCE_ALARM",
    "DELETE_ALARM",
    "UNLOOP_MUSIC",
    "UPDATE_ALARM",
    "CREATE_PLAYLIST_MUSIC",
    "DELETE_REMINDER",
    "CREATE_REMINDER",
    "ANSWER_CALL",
    "SET_AVAILABLE",
    "SWITCH_CALL",
    "UPDATE_REMINDER_DATE_TIME",
    "GET_REMINDER",
    "GET_REMINDER_DATE_TIME",
    "UPDATE_REMINDER_TODO",
    "IGNORE_CALL",
    "GET_INFO_CONTACT",
    "GET_EMPLOYER",
    "GET_EMPLOYMENT_TIME",
    "GET_EDUCATION_TIME",
    "GET_UNDERGRAD",
    "GET_AGE",
    "GET_EDUCATION_DEGREE",
    "GET_INFO_RECIPES",
    "GET_RECIPES",
    "IS_TRUE_RECIPES",
    "PREVIOUS_TRACK_MUSIC",
    "QUESTION_NEWS",
    "GET_DETAILS_NEWS",
    "ADD_TIME_TIMER",
    "LIKE_MUSIC",
    "ADD_TO_PLAYLIST_MUSIC",
    "QUESTION_MUSIC",
    "SKIP_TRACK_MUSIC",
    "LOOP_MUSIC",
    "PAUSE_MUSIC",
    "GET_TRACK_INFO_MUSIC",
    "DISLIKE_MUSIC",
    "RESTART_TIMER",
    "DELETE_TIMER",
    "DELETE_PLAYLIST_MUSIC",
    "REMOVE_FROM_PLAYLIST_MUSIC",
    "REWIND_MUSIC",
    "REPLAY_MUSIC",
    "START_SHUFFLE_MUSIC",
    "STOP_MUSIC",
    "FAST_FORWARD_MUSIC",
    "FOLLOW_MUSIC",
    "SET_RSVP_NO",
    "GET_ATTENDEE_EVENT",
    "GET_SUNRISE",
    "HOLD_CALL",
    "GET_CALL_CONTACT",
    "GET_REMINDER_AMOUNT",
    "HELP_REMINDER",
    "GET_REMINDER_LOCATION",
    "UPDATE_REMINDER",
    "UPDATE_REMINDER_LOCATION",
    "REPEAT_ALL_OFF_MUSIC",
    "GET_CONTACT_METHOD",
    "GET_JOB",
    "GET_LIFE_EVENT",
    "GET_GENDER",
    "GET_MAJOR",
    "GET_MUTUAL_FRIENDS",
    "GET_LYRICS_MUSIC",
    "CANCEL_MESSAGE",
    "RESUME_CALL",
    "REPEAT_ALL_MUSIC",
    "RESUME_MUSIC",
    "GET_LIFE_EVENT_TIME",
    "GET_LANGUAGE",
    "MERGE_CALL",
    "GET_AIRQUALITY",
]